/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.p4java.extension.operation;

import java.util.EnumSet;
import java.util.Map;

import com.perforce.p4java.extension.command.PerforceDomain;
import com.perforce.p4java.extension.command.RequestImpl;
import com.perforce.p4java.extension.exception.ServerAccessException;
import com.perforce.p4java.extension.exception.ServerConnectionException;
import com.perforce.p4java.extension.exception.ServerErrorException;
import com.perforce.p4java.extension.path.PathStringFormat;
import com.perforce.p4java.extension.path.PerforcePath;
import com.perforce.p4java.extension.server.Server;

public class SizesCommand {

    public static String tagDepotFile = "depotFile";
    public static String tagFileSize = "fileSize";
    public static String tagRevision = "rev";

    public static Map<String, Object>[] getAllSizes(final String path,
	    final Server server) throws ServerConnectionException,
	    ServerAccessException, ServerErrorException {
	return server.execute(new RequestImpl() {

	    @Override
	    public String getCommand() {
		return PerforceDomain.SIZES.getCommand();
	    }

	    @Override
	    public String[] getCommandArgs() {
		return new String[] { "-a", path };
	    }

	});
    }

    public static Map<String, Object>[] getSizes(final PerforcePath path,
	    final Server server) throws ServerConnectionException,
	    ServerAccessException, ServerErrorException {
	return server.execute(new RequestImpl() {

	    @Override
	    public String getCommand() {
		return PerforceDomain.SIZES.getCommand();
	    }

	    @Override
	    public String[] getCommandArgs() {
		return new String[] { path.asString((EnumSet
			.of(PathStringFormat.ESCAPE))) };
	    }

	});
    }

    public static Map<String, Object>[] getSizesAtPath(final PerforcePath path,
	    final Server server) throws ServerConnectionException,
	    ServerAccessException, ServerErrorException {
	return server.execute(new RequestImpl() {
	    @Override
	    public String getCommand() {
		return PerforceDomain.SIZES.getCommand();
	    }

	    @Override
	    public String[] getCommandArgs() {
		String escaped = path.asString(EnumSet
			.of(PathStringFormat.ESCAPE));
		escaped = escaped + "*"; // DO NOT ENQUOTE
					 // THIS!!!
		String[] args = { escaped };
		return args;
	    }

	});
    }
}
